/******************************************************************************
  (c) Copyright 2002,2003, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: SAXPipelineTransreptor.java,v $
  Version:       $Name:  $ $Revision: 1.1 $
  Last Modified: $Date: 2008/05/02 08:20:20 $
 *****************************************************************************/

package org.netkernel.xml.transrepresentation;


import org.apache.xml.serializer.OutputPropertiesFactory;
import org.apache.xml.serializer.Serializer;
import org.apache.xml.serializer.SerializerFactory;
import org.apache.xml.utils.DOMBuilder;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.representation.IRepSAXHead;
import org.netkernel.xml.representation.RepSAXHead;
import org.netkernel.xml.util.NKFEntityResolver;
import org.netkernel.xml.util.XMLUtils;
import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.*;
import java.util.Properties;

import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXTransformerFactory;

/**
 * Serialise SAX pipeline to a binary stream
 * @author  pjr
 */
public class SAXPipelineTransreptor extends StandardTransreptorImpl implements ErrorHandler
{	public static final String META_ENCODING="encoding";
	
	public SAXPipelineTransreptor()
	{	this.declareThreadSafe();
		this.declareName("SAXTransreptor");
		this.declareDescription("Transrept SAX pipeline to binary stream, DOM, String");
		this.declareFromRepresentation(XMLFilter.class);
		this.declareToRepresentation(IReadableBinaryStreamRepresentation.class);
		this.declareToRepresentation(Document.class);
		this.declareToRepresentation(String.class);
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	INKFResponseReadOnly resp=aContext.getThisRequest().getPrimaryAsResponse();
		XMLFilter tailFilter=(XMLFilter)resp.getRepresentation();
		tailFilter.setErrorHandler(this);
		tailFilter.setEntityResolver(new NKFEntityResolver(aContext));
		Class to=aContext.getThisRequest().getRepresentationClass();
		boolean toString=String.class.isAssignableFrom(to);
		boolean toStream= IReadableBinaryStreamRepresentation.class.isAssignableFrom(to); 
		if( toStream || toString)
		{	String mimetype = resp.getMimeType();
			String serializationType="xml";			//default to xml
			if(mimetype.equals("text/html"))
			{	serializationType="html";			
			}
			else if(mimetype.equals("text/plain"))
			{	serializationType="text";				
			}
			Properties p=OutputPropertiesFactory.getDefaultMethodProperties(serializationType);
			Serializer s=SerializerFactory.getSerializer(p);
			ByteArrayOutputStream baos=new ByteArrayOutputStream(1024);
			s.setOutputStream(baos);
			ContentHandler ch = s.asContentHandler();
			tailFilter.setContentHandler(ch);
			IRepSAXHead head=getSAXHeadFilter(tailFilter);
			head.trigger();
			Object result=null;
			if(toString)
			{	result=baos.toString();				
			}
			else
			{	result=new ByteArrayRepresentation(baos.toByteArray());				
			}
			aContext.createResponseFrom(result);
		}
		else
		{	Document result=XMLUtils.newDocument();
			DOMBuilder db=new DOMBuilder(result);
			tailFilter.setContentHandler(db);
			IRepSAXHead head=getSAXHeadFilter(tailFilter);
			head.trigger();
			aContext.createResponseFrom(result);
		}
	}
	
	public static IRepSAXHead getSAXHeadFilter(XMLFilter r) throws NKFException
	{	if(r instanceof RepSAXHead)
		{	return (IRepSAXHead)r;
		}
		XMLFilter head=r;
		XMLFilter parent=(XMLFilter)r.getParent();
		while(parent!=null)
		{	head=parent;
			if(head instanceof RepSAXHead)
			{	break;				
			}
			parent=(XMLFilter)head.getParent();
		}
		if(parent==null && !(head instanceof RepSAXHead))
		{	throw new NKFException("Pipeline does not have a RepSAXHead representation at its head");	
		}
		return (IRepSAXHead)head;	
	}

	public void error(SAXParseException exception) throws SAXException
	{	throw exception;		
	}

	public void fatalError(SAXParseException exception) throws SAXException
	{	throw exception;		
	}

	public void warning(SAXParseException exception) throws SAXException
	{	throw exception;		
	}

}